﻿using System;
using System.Collections.Generic;

namespace AdvBusinessServices
{
    public static class DataUtility
    {
        private static Dictionary<int, string> _scriptList = new Dictionary<int, string>();
        private static Dictionary<string, Order> _orderList = new Dictionary<string,Order>();
        private static Dictionary<int, Customer> _customerList = new Dictionary<int,Customer>();
        private static Dictionary<int, SecurityQuestion> _questionList = new Dictionary<int, SecurityQuestion>();
        private static Dictionary<string, User> _userList = new Dictionary<string, User>();

        public static Customer GetCustomerByID(int customerID)
        {
            return _customerList[customerID];
        }
        public static User GetUserByID(string userID)
        {
            return _userList[userID];
        }
        public static List<SecurityQuestion> GetSecurityQuestions(string orderNumber)
        {
            Random rnd = new Random();
            int numQuestions = rnd.Next(1, 5); //this will return a random number from 1 to 4
            List<SecurityQuestion> questions = new List<SecurityQuestion>();
            Order order = GetOrderByID(orderNumber);
            Customer customer = GetCustomerByID(order.CustomerID);

            for (int i = 0; i < numQuestions; i++)
            {
                int questionKey = i + 1;
                SecurityQuestion q = (SecurityQuestion)_questionList[questionKey].Clone();
               
                switch( q.AnswerType)
                {
                    case 1:
                         q.Answer =  customer.PhoneNumber;  
                        break;
                    case 2:
                        q.Answer = order.ShipToAddress;
                        break;
                    case 3:
                        q.Answer = "$" + order.OrderTotal.ToString();
                        break;
                    case 4:
                        q.Answer = order.CustomerID.ToString();
                        break;
                    default:
                        break;
                }
                questions.Add(q);
            }
            return questions;
        }

        public static Order GetOrderByID(string orderNumber)
        {
            return _orderList[orderNumber];
        }

        public static string GetAgentScript(int scriptType)
        {
            return _scriptList[scriptType];
        }

        static DataUtility()
        {
            LoadTestData();
        }

        private static void LoadTestData()
        {
            string script = @"Hello Mr. {0}, thank you so much for calling us today. How is your day? [listen to " +
                            @"the customer's response] How can I help you? [listen to the response] I will be glad " +
                            @"to help you. First, for the security purpose, I need to ask you a few questions...";

            
            _userList.Add("jc55", new User(){ UserID="jc55",RepNumber="jc55",FirstName="Nancy",LastName="Davolio", Role="Agent", PhoneExtension=5467});
            _userList.Add("bj55", new User() { UserID = "bj55", RepNumber = "bj55", FirstName = "Andrew", LastName = "Fuller", Role = "Agent", PhoneExtension = 3457 });
            _userList.Add("wl55", new User() { UserID = "wl55", RepNumber = "wl55", FirstName = "Janet", LastName = "Leverling", Role = "Supervisor", PhoneExtension = 3355 });
            
            //build scripts
            _scriptList.Add(1, script);

            //build customer list
            _customerList.Add(12334, new Customer() { CustomerID = 12334, FirstName = "Margaret", LastName = "Peacock", PhoneNumber = "(206) 555-8122", Address = "4110 Old Redmond Rd.	Redmond, WA	98052, USA" });
            _customerList.Add(98673, new Customer() { CustomerID = 98673, FirstName = "Steven", LastName = "Buchanan", PhoneNumber = "(71) 555-4848", Address = "14 Garrett Hill, London, SW1 8JR	UK" });
            _customerList.Add(58764, new Customer() { CustomerID = 58764, FirstName = "Michael", LastName = "Suyama", PhoneNumber = "(71) 555-7773", Address = "Coventry House Miner Rd., London, EC2 7JR	UK" });
            _customerList.Add(27334, new Customer() { CustomerID = 27334, FirstName = "Robert", LastName = "King", PhoneNumber = "(71) 555-5598", Address = "Edgeham Hollow Winchester Way, London,	RG1 9SP	UK" });

            //build order list
            _orderList.Add("39688", new Order() { CustomerID = 12334, Description = "Order for small widgets", OrderNumber = "39688", OrderTotal = 99.57, ShipToAddress = "4110 Old Redmond Rd.	Redmond, WA	98052, USA" });
            _orderList.Add("57754", new Order() { CustomerID = 98673, Description = "Order for a mountain bike", OrderNumber = "57754", OrderTotal = 186.86, ShipToAddress = "14 Garrett Hill, London, SW1 8JR	UK" });
            _orderList.Add("58967", new Order() { CustomerID = 58764, Description = "Order for a tricycle", OrderNumber = "58967", OrderTotal = 148.47, ShipToAddress = "Coventry House Miner Rd., London, EC2 7JR	UK" });
            _orderList.Add("76948", new Order() { CustomerID = 27334, Description = "Order for bicyle tires", OrderNumber = "76948", OrderTotal = 10.54, ShipToAddress = "Edgeham Hollow Winchester Way, London, RG1 9SP	UK" });

            //build question list
            _questionList.Add(1, new SecurityQuestion() { QuestionID = 1, Question = "what is your phone number?", AnswerType=1, Answer="{0}" });
            _questionList.Add(2, new SecurityQuestion() { QuestionID = 2, Question = "what is your order address?", AnswerType = 2, Answer = "{0}" });
            _questionList.Add(3, new SecurityQuestion() { QuestionID = 3, Question = "what is your order amount?", AnswerType = 3, Answer = "{0}" });
            _questionList.Add(4, new SecurityQuestion() { QuestionID = 4, Question = "what is your account number?", AnswerType = 4, Answer = "{0}" });
        }
    }
}