﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;

namespace AdvBusinessServices
{
    [ServiceContract]
    public interface ICallService
    {
        [OperationContract]
        AgentScript GetAgentScript(string orderNumber);
        [OperationContract]
        OrderInfo GetOrderDetails(string orderNumber);
    }

    [ServiceContract]
    public interface IUserProfile    
    {
        [OperationContract]
        User GetUser(string userID);
    }

    [DataContract]
    public class MessageEntity
    {
        [DataMember]
        double ServiceVersion = 1.1;
    }
    [DataContract]
    public class AgentScript:MessageEntity
    {
        [DataMember]
        public int CustomerID;
        [DataMember]
        public string Greeting;
        [DataMember]
        public int MinRequiredAnswers;
        [DataMember]
        public List<SecurityQuestion> QuestionList;
    }

    [DataContract]
    public class SecurityQuestion:MessageEntity
    {
        [DataMember]
        public int QuestionID;
        [DataMember]
        public string Question;
        [DataMember]
        public int AnswerType;
        [DataMember]
        public string Answer;
        [DataMember]
        public bool IsAnswered;
        public SecurityQuestion Clone() {return (SecurityQuestion) this.MemberwiseClone();}
    }

    [DataContract]
    public class User:MessageEntity
    {
        [DataMember]
        public string UserID;
        [DataMember]
        public string FirstName;
        [DataMember]
        public string LastName;
        [DataMember]
        public string RepNumber;
        [DataMember]
        public string Role;
        [DataMember]
        public int PhoneExtension;
    }

    [DataContract]
    public class Order:MessageEntity
    {
        [DataMember]
        public int CustomerID;
        [DataMember]
        public string OrderNumber;
        [DataMember]
        public string Description;
        [DataMember]
        public double OrderTotal;
        [DataMember]
        public string ShipToAddress;
        
    }

    [DataContract]
    public class Customer:MessageEntity
    {
        [DataMember]
        public int CustomerID;
        [DataMember]
        public string FirstName;
        [DataMember]
        public string LastName;
        [DataMember]
        public string PhoneNumber;
        [DataMember]
        public string Address;
    }

    [DataContract]
    public class OrderInfo:MessageEntity
    {
        [DataMember]
        public Order Order;
        [DataMember]
        public Customer Customer;
    }


}
