﻿using System;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using AdvCallCenterClient.CallBusinessProxy;
using AdvClientLibrary;
using SilverlightPopups;
using TcpMessages;
using ClientEntities = AdvClientLibrary.ClientEntities;
using System.Net;
using System.IO;
using System.Reflection;

namespace AdvCallCenterClient
{
    public partial class Page : UserControl
    {
        private UserControl _orderDetailContol;
        private SLManualResetEvent _orderDetailDownloadHandle = new SLManualResetEvent(false);
        public Page()
        {
            InitializeComponent();
            ClientGlobals.socketClient = new CallNetworkClient();
            ClientGlobals.socketClient.OnReceive += new EventHandler<ReceiveArgs>(ReceiveCallback);
            ClientGlobals.socketClient.OnConnectError += new EventHandler<EventArgs>(_socketClient_OnConnectError);
            ClientGlobals.socketClient.Connect(4530);
            ClientGlobals.networkPopup = new CommunicationPopup(this, "Send in progress...");
            ClientGlobals.notifyCallPopup = new NotifyCallPopup(this, "Incoming call...");
            ClientGlobals.notifyCallPopup.OnAccept += new EventHandler<EventArgs>(_notifyCallPopup_OnAccept);
            ClientGlobals.notifyCallPopup.OnReject += new EventHandler<EventArgs>(_notifyCallPopup_OnReject);
            ClientGlobals.repInfo = new ClientEntities.CallRep();
        }

        void _notifyCallPopup_OnReject(object sender, EventArgs e)
        {
            RepKeyMessage rejectMsg = new RepKeyMessage(MessageAction.Reject);
            rejectMsg.RepNumber = ClientGlobals.currentUser.RepNumber;
            ClientGlobals.socketClient.SendAsync(rejectMsg);
        }

        void _notifyCallPopup_OnAccept(object sender, EventArgs e)
        {
            RepKeyMessage acceptMsg = new RepKeyMessage(MessageAction.Accept);
            acceptMsg.RepNumber = ClientGlobals.currentUser.RepNumber;
            ClientGlobals.socketClient.SendAsync(acceptMsg);
            this.borderCallProgressView.DataContext = ClientGlobals.callInfo;
            ICallService callService = new CallServiceClient();
            IAsyncResult result = callService.BeginGetAgentScript(ClientGlobals.callInfo.OrderNumber, GetAgentScriptCallback, callService);
            //do a preemptive download of user control
            ThreadPool.QueueUserWorkItem(ExecuteControlDownload);
            //do a preemptive download of the order information
            ThreadPool.QueueUserWorkItem(ExecuteGetOrderDetails, ClientGlobals.callInfo.OrderNumber);

        }

        void GetAgentScriptCallback(IAsyncResult asyncReseult)
        {  
             ICallService callService = asyncReseult.AsyncState as ICallService;
             CallBusinessProxy.AgentScript svcOutputAgentScript = callService.EndGetAgentScript(asyncReseult);
             ClientEntityTranslator astobas = SvcScriptToClientScript.entityTranslator;
             ClientEntities.AgentScript currentAgentScript = astobas.ToClientEntity(svcOutputAgentScript) as ClientEntities.AgentScript;
             Interlocked.Exchange<ClientEntities.AgentScript>(ref ClientGlobals.currentAgentScript, currentAgentScript);
             if (this.Dispatcher.CheckAccess())
             {
                 this.borderAgentScript.DataContext = ClientGlobals.currentAgentScript;
                 //this.gridSecurityQuestions.DataContext = ClientGlobals.agentScript.QuestionList;
                 this.gridSecurityQuestions.ItemsSource = ClientGlobals.currentAgentScript.QuestionList;
                 this.borderWaitView.Visibility = Visibility.Collapsed;
                 this.hlEndCall.Visibility = Visibility.Collapsed;
                 this.borderCallProgressView.Visibility = Visibility.Visible;
                 this.gridSecurityQuestions.Visibility = Visibility.Visible;
                 this.borderAgentScript.Visibility = Visibility.Visible;
                 this.hlVerifyContinue.Visibility = Visibility.Visible;
             }
             else
             {
                 this.Dispatcher.BeginInvoke(
                     delegate()
                     {
                         this.borderAgentScript.DataContext = ClientGlobals.currentAgentScript;
                         this.gridSecurityQuestions.DataContext = ClientGlobals.currentAgentScript.QuestionList;
                         //this.gridSecurityQuestions.ItemsSource = ClientGlobals.agentScript.QuestionList;
                         this.borderWaitView.Visibility = Visibility.Collapsed;
                         this.hlEndCall.Visibility = Visibility.Collapsed;
                         this.borderCallProgressView.Visibility = Visibility.Visible;
                         this.gridSecurityQuestions.Visibility = Visibility.Visible;
                         this.borderAgentScript.Visibility = Visibility.Visible;
                         this.hlVerifyContinue.Visibility = Visibility.Visible;

                     });
             }
        }

        void _socketClient_OnConnectError(object sender, EventArgs e)
        {
            this.tbStatus.Dispatcher.BeginInvoke(delegate() { this.tbStatus.Text = "Connect error..."; });
        }

        void ReceiveCallback(object sender, ReceiveArgs e)
        {
            NetworkMessage msg = e.Result;
            ProcessMessage(msg);
        }

        private void buttonSubmit_Click(object sender, RoutedEventArgs e)
        {
            ClientGlobals.repInfo.ShiftStartTime = DateTime.Now;
            RegisterMessage registerMsg = new RegisterMessage(MessageAction.Register);
            registerMsg.RepName = ClientGlobals.currentUser.Name;
            registerMsg.RepNumber = ClientGlobals.currentUser.RepNumber;
            registerMsg.PhoneExtension = ClientGlobals.currentUser.PhoneExtension;
            ClientGlobals.networkPopup.ShowDialog(true);
            ClientGlobals.socketClient.SendAsync(registerMsg);
        }

        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
            this.ctrlLoginView.Visibility = Visibility.Visible;

        }
        void ProcessMessage(NetworkMessage msg)
        {
            //switch(msg.GetMessageType())
            switch(msg.Action)
            {
                //case MessageType.RegisterResponse:
                case MessageAction.RegisterResponse:
                    RegisterResponse respMsg = msg as RegisterResponse;
                    //check if the code is executing on the UI thread
                    //the if is unncessary as the code always executes in the background thread
                    if (!this.Dispatcher.CheckAccess())
                    {
                        //the following Sleep is only meant to show the popup long enough
                        //during demos; remove for real use
                        System.Threading.Thread.Sleep(1000);
                        this.Dispatcher.BeginInvoke(
                            delegate()
                            {
                                ClientGlobals.networkPopup.CloseDialog();
                                this.registrationView.Visibility = Visibility.Collapsed;
                                this.callView.Visibility = Visibility.Visible;
                                this.borderWaitView.Visibility = Visibility.Visible;
                                //do databinding here
                                this.tbRepDisplayName.Text = this.txRepName.Text;
                                this.tbRepDisplayNumber.Text = respMsg.RepNumber;
                                this.tbCallServerName.Text = respMsg.CallManagerServerName;
                                DateTime ts = respMsg.RegistrationTimestamp;
                                this.tbCallStartTime.Text = string.Format("{0:D2}/{1:D2}/{2:D4} {3:D2}:{4:D2}:{5:D2}", ts.Month, ts.Day, ts.Year, ts.Hour, ts.Minute, ts.Second);
                            }
                        );
                    }
                    break;
                //case MessageType.Call:
                case MessageAction.Call:
                    CallMessage callMsg = msg as CallMessage;
                    ClientEntities.PhoneCall currentCallInfo = new ClientEntities.PhoneCall();
                    currentCallInfo.Name = callMsg.CallerName;
                    currentCallInfo.PhoneNumber = callMsg.CallerPhone;
                    currentCallInfo.OrderNumber = callMsg.IvrData;
                    Interlocked.Exchange<ClientEntities.PhoneCall>(ref ClientGlobals.callInfo, currentCallInfo);
                    if (!this.Dispatcher.CheckAccess())
                    {
                        this.Dispatcher.BeginInvoke(
                            delegate() { ClientGlobals.notifyCallPopup.ShowDialog(true); });
                    }
                    break;
                default:
                    break;
            }
        }

        private void hlVerifyContinue_Click(object sender, RoutedEventArgs e)
        {
            this.gridSecurityQuestions.Visibility = Visibility.Collapsed;
            //if the control is not instandiated, add it to the control tree
            if (_orderDetailContol== null)
            {
                _orderDetailContol = ClientGlobals.advOrderControls_dll.CreateInstance("AdvOrderClientControls.OrderDetail") as UserControl;
                spCallProgressPanel.Children.Add(_orderDetailContol);
            }
            _orderDetailContol.DataContext = ClientGlobals.currentOrder;
            borderCall.Visibility = Visibility.Collapsed;
            borderAgentScript.Visibility = Visibility.Collapsed;
            hlVerifyContinue.Visibility = Visibility.Collapsed;
            hlEndCall.Visibility = Visibility.Visible;
            _orderDetailContol.Visibility = Visibility.Visible;
         
        }

        private void ExecuteControlDownload(object state)
        {
            WebClient webClient = new WebClient();
            webClient.OpenReadCompleted += new OpenReadCompletedEventHandler(OrderDetailControlDownloadCallback);
            webClient.OpenReadAsync(new Uri("/ClientBin/AdvOrderClientControls.dll", UriKind.Relative));
        }

        void OrderDetailControlDownloadCallback(object sender, OpenReadCompletedEventArgs e)
        {
            try
            {
                //if there is a problem with the download the following statement will through an exception
                Stream downloadedStream = e.Result;
                this.Dispatcher.BeginInvoke(delegate()
                {
                    Assembly asm = ResourceUtility.GetAssembly(downloadedStream);
                    Interlocked.Exchange<Assembly>(ref ClientGlobals.advOrderControls_dll, asm );
                });
            }
            catch
            {
                throw new SLApplicationException("Download of OrderDetail control failed");
            }
        }

        private void ExecuteGetOrderDetails(object state)
        {
            CallServiceClient client = new CallServiceClient();
            string orderNumber = state as string;
            client.GetOrderDetailsCompleted += new EventHandler<GetOrderDetailsCompletedEventArgs>(GetOrderDetailsCompletedCallback);
            client.GetOrderDetailsAsync(orderNumber);
            this._orderDetailDownloadHandle.Reset();
            this._orderDetailDownloadHandle.WaitOne();
            ClientEntityTranslator oito = SvcOrderToClientOrder.entityTranslator;
            ClientEntities.Order currentOrder = oito.ToClientEntity(ClientGlobals.serviceOutputOrder)as ClientEntities.Order;
            Interlocked.Exchange<ClientEntities.Order>(ref ClientGlobals.currentOrder, currentOrder);
        }

        void GetOrderDetailsCompletedCallback(object sender, GetOrderDetailsCompletedEventArgs e)
        {
            Interlocked.Exchange<OrderInfo>(ref ClientGlobals.serviceOutputOrder, e.Result);
            //this._orderDetailDownloadHandle.Set();
            this._orderDetailDownloadHandle.Set();

        }

        private void hlEndCall_Click(object sender, RoutedEventArgs e)
        {
            borderCallProgressView.Visibility = Visibility.Collapsed;
            _orderDetailContol.Visibility = Visibility.Collapsed;
            borderWaitView.Visibility = Visibility.Visible;
            ClientGlobals.repInfo.NoOfCallsServiced++;
            this.tbNumCalls.Text = ClientGlobals.repInfo.NoOfCallsServiced.ToString();
            RepKeyMessage finishCallMsg = new RepKeyMessage(MessageAction.FinishCall);
            finishCallMsg.RepNumber = ClientGlobals.currentUser.RepNumber;
            ClientGlobals.callInfo = null;
            ClientGlobals.currentAgentScript = null;
            ClientGlobals.currentOrder = null;
            ClientGlobals.serviceOutputOrder = null;
            ClientGlobals.socketClient.SendAsync(finishCallMsg);
        }

        private void ctrlLoginView_OnSuccessfulLogin(object sender, EventArgs e)
        {
            Login login = sender as Login;
            CallBusinessProxy.UserProfileClient userProfile = new CallBusinessProxy.UserProfileClient();
            userProfile.GetUserCompleted += new EventHandler<GetUserCompletedEventArgs>(GetUserCompletedCallback);
            userProfile.GetUserAsync(login.txRepID.Text);
        }

        void GetUserCompletedCallback(object sender, GetUserCompletedEventArgs e)
        {
            CallBusinessProxy.User serviceOutputUser = e.Result;
            ClientEntityTranslator utbu = SvcUserToClientUser.entityTranslator;
            ClientEntities.User currentUser = utbu.ToClientEntity(serviceOutputUser) as ClientEntities.User;
            Interlocked.Exchange<ClientEntities.User>(ref ClientGlobals.currentUser, currentUser);
            if (this.Dispatcher.CheckAccess())
            {
                this.registrationView.DataContext = ClientGlobals.currentUser;
                this.ctrlLoginView.Visibility = Visibility.Collapsed;
                this.registrationView.Visibility = Visibility.Visible;
                this.buttonSubmit.IsEnabled = true;
            }
            else
            {
                this.Dispatcher.BeginInvoke(delegate()
                {
                    this.registrationView.DataContext = ClientGlobals.currentUser;
                    this.ctrlLoginView.Visibility = Visibility.Collapsed;
                    this.registrationView.Visibility = Visibility.Visible;
                    this.buttonSubmit.IsEnabled = true;
                });
            }
        }
    }
}
