﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using TcpMessages;

namespace AdvCallManager
{
    /// <summary>
    /// CallDispatcheer manage a list of RepConnection objects one per each registered
    /// call center rep. 
    /// </summary>
    class CallDispatcher
    {
        private Dictionary<string, CallRep> _repList = new Dictionary<string, CallRep>();
        private CallQueue _callQueue = new CallQueue();
        public CallDispatcher()
        {
            //register call queue listener
            _callQueue.OnCallReceived += new EventHandler<EventArgs>(DispatchCall);

        }
        void DispatchCall(Object sender, EventArgs args)
        {
            PhoneCall customerCall = _callQueue.Dequeue();
            CallRep rep = GetAvailableRep();
            if (rep == null)
            {
                Console.WriteLine("Rep is not available...");
                return;
            }
            rep.Locked = true;
            rep.LockTime = DateTime.Now;
            CallMessage callMsg = new CallMessage(MessageAction.Call);
            callMsg.CallerName = customerCall.Name;
            callMsg.CallerPhone = customerCall.PhoneNumber;
            callMsg.IvrData = customerCall.OrderNumber;
            rep.RepConnection.SendAsync(callMsg);
            Console.WriteLine("forwarding call to the rep...");
        }
        public CallRep GetAvailableRep()
        {
            foreach (string key in _repList.Keys)
            {
                CallRep rep = _repList[key];              
                if (!rep.Available) continue;
                if (rep.Locked) continue;
                //don't dispatch if a rep just rejected the call
                if (!(rep.LastRejectTime == DateTime.MaxValue))
                {
                    //rep recently rejected the call. let us see if 2 minutes elapsed since 
                    if (rep.LastRejectTime.AddMinutes(2.0) > DateTime.Now)
                    {
                        //time has not elapsed
                        continue;
                    }
                }
                return rep;
            }
            return null;
        }


        public Dictionary<string, CallRep> RepList
        {
            get { return _repList; }
        }

        public CallQueue CustomerCallQueue
        {
            get { return _callQueue; }
        }
        public void CleanUp()
        {
            foreach (string key in _repList.Keys)
            {
                CallRep rep = _repList[key];
                rep.RepConnection.AcceptedSocket.Shutdown(SocketShutdown.Both);
                rep.RepConnection.AcceptedSocket.Close();
            }
        }
    }
}
