﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Threading;

namespace AdvClientLibrary
{
    public class SLManualResetEvent
    {
        private ManualResetEvent _manualResetEvent;
        public SLManualResetEvent()
        {
            _manualResetEvent = new ManualResetEvent(false);
        }
        public SLManualResetEvent(bool initialState)
        {
            _manualResetEvent = new ManualResetEvent(initialState);
        }
        public bool CanBlock
        {
            get
            {
                if (ApplicationHelpers.IsUiThread())
                {
                    return false; //we don't want to block the UI thread;
                }
                else
                {
                    return true; //we are on the worker thread 
                }
            }
        }
        public bool WaitOne()
        {
            if (CanBlock)
                return _manualResetEvent.WaitOne();
            else
                throw new UiThreadBlockException("Can't block UI thread");
        }

        public bool WaitOne(int milliSeconds)
        {
            if (CanBlock)
                return _manualResetEvent.WaitOne(milliSeconds);
            else
                throw new UiThreadBlockException("Can't block UI thread");
        }

        public bool WaitOne(TimeSpan timeout)
        {
            if (CanBlock)
                return _manualResetEvent.WaitOne(timeout);
            else
                throw new UiThreadBlockException("Can't block UI thread");
        }
        public bool Set()
        {
            return _manualResetEvent.Set();
        }
        public bool Reset()
        {
            return _manualResetEvent.Reset();
        }
    }

    public class UiThreadBlockException : Exception
    {
        public UiThreadBlockException() : base() { }
        public UiThreadBlockException(string message) : base(message) { }
        public UiThreadBlockException(string message, Exception innerException)
            : base(message, innerException) { }
    }
}