﻿using System;
using System.Windows;
using System.Reflection;
using System.IO;
using System.Windows.Resources;
using System.Xml;
using System.Xml.Linq;
using System.Windows.Markup;

namespace AdvClientLibrary
{
    public class ResourceUtility
    {
        public static Assembly GetAssembly(string assemblyName, Stream packageStream)
        {
            StreamResourceInfo srInfo =
            Application.GetResourceStream(
                     new StreamResourceInfo(packageStream, "application/binary"),
                     new Uri(assemblyName, UriKind.Relative));
            return GetAssembly(srInfo.Stream);
        }

        public static Assembly GetAssembly(Stream assemblyStream)
        {
            AssemblyPart assemblyPart = new AssemblyPart();
            return assemblyPart.Load(assemblyStream);
        }

        public static XElement GetXmlDocument(Stream xmlStream)
        {
            XmlReader reader = XmlReader.Create(xmlStream);
            XElement element = XElement.Load(reader);
            return element;
        }
        public static XElement GetXmlDocument(string fileName, Stream packageStream)
        {
            Stream rs = GetStream(fileName, "application/xml", packageStream);
            XmlReader reader = XmlReader.Create(rs);
            XElement element = XElement.Load(reader);
            return element;
        }

        public static Stream GetStream (string fileName, string mimeType, Stream packageStream)
        {
            StreamResourceInfo sri = new StreamResourceInfo(packageStream, mimeType);
            return Application.GetResourceStream(sri, new Uri(fileName, UriKind.Relative)).Stream;
        }
        public static XElement GetXmlDocumentFromXap(string fileName)
        {
            XmlReaderSettings settings = new XmlReaderSettings();
            settings.XmlResolver = new XmlXapResolver();
            XmlReader reader = XmlReader.Create(fileName);
            XElement element = XElement.Load(reader);
            return element;
        }
        //gets the UIElement from the default package
        public static UIElement GetUIElementFromXaml(string xamlFileName)
        {
            StreamResourceInfo streamInfo = Application.GetResourceStream(new Uri(xamlFileName, UriKind.Relative));
            string xaml = new StreamReader(streamInfo.Stream).ReadToEnd();
            UIElement uiElement = null;
            try
            {
                uiElement = (UIElement)XamlReader.Load(xaml);
            }
            catch
            {
                throw new SLApplicationException(string.Format("Can't create UIElement from {0}", xamlFileName));
            }
            return uiElement;
        }
    }

}

