﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using TcpMessages;

namespace AdvPolicyServer
{
    class TcpClientConnection
    {
        private PolicyReceiveBuffer _receiveBuffer;
        private Socket _acceptedSocket;
        private SocketAsyncEventArgs _receiveSocketArgs;
        private byte[] _policyBuffer;
        public PolicyReceiveBuffer ReceiveBuffer
        {
            get { return _receiveBuffer; }
            set { _receiveBuffer = value; }
        }
        public TcpClientConnection(Socket acceptedSocket, byte[] policyBuffer)
        {
            this._acceptedSocket = acceptedSocket;
            this._policyBuffer = policyBuffer;
            this.ReceiveBuffer = new PolicyReceiveBuffer(1024);
            this._receiveSocketArgs = new SocketAsyncEventArgs();
            this._receiveSocketArgs.SetBuffer(_receiveBuffer.Buffer, 0, _receiveBuffer.Buffer.Length);
            _receiveSocketArgs.Completed += new EventHandler<SocketAsyncEventArgs>(ReceiveCallback);
        }
        public Socket AcceptedSocket
        {
            get { return _acceptedSocket; }
        }

        public void ReceiveAsync()
        {
            ReceiveAsync(_receiveSocketArgs);
        }

        private void ReceiveAsync(SocketAsyncEventArgs socketAsyncEventArgs)
        {
            if (!_acceptedSocket.ReceiveAsync(socketAsyncEventArgs))
            {
                ReceiveCallback(_acceptedSocket, socketAsyncEventArgs);
            }
        }

        void ReceiveCallback(object sender, SocketAsyncEventArgs e)
        {
            if (e.SocketError != SocketError.Success)
            {
                return;
            }
            _receiveBuffer.Offset += e.BytesTransferred;
            if (_receiveBuffer.IsMessageReceived())
            {
                SendPolicy();
            }
            else
            {
                //adjust the buffer pointer
                e.SetBuffer(_receiveBuffer.Offset, _receiveBuffer.Remaining);
                ReceiveAsync(_receiveSocketArgs);
            }            
        }
        public void SendPolicy()
        {
            SendAsync(_policyBuffer);
        }
        public void SendAsync(byte[] sendBytes)
        {
            SocketAsyncEventArgs sendSocketArgs = new SocketAsyncEventArgs();
            sendSocketArgs.SetBuffer(sendBytes, 0, sendBytes.Length);
            sendSocketArgs.Completed += new EventHandler<SocketAsyncEventArgs>(SendCallback);
            SendAsync(sendSocketArgs);
        }

        private void SendAsync(SocketAsyncEventArgs sendSocketArgs)
        {
            if (!_acceptedSocket.SendAsync(sendSocketArgs))
            {
                //call is completed synchronously
                SendCallback(_acceptedSocket, sendSocketArgs);
            }
        }

        void SendCallback(object sender, SocketAsyncEventArgs e)
        {
            if (e.BytesTransferred < e.Count)
            {
                //some more bytes to send
                e.SetBuffer(e.BytesTransferred, e.Count - e.BytesTransferred);
                SendAsync(e);
            }
            else
            {
                //we no longer need the connection
                this.AcceptedSocket.Close();
            }
        }
    }
}
