﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.IO;

namespace AdvPolicyServer
{
    class TcpPolicyServer
    {
        private Socket _listener;
        private byte[] _policyBuffer;
        public static readonly string PolicyFileName = "clientaccesspolicy.xml";
        private SocketAsyncEventArgs _socketAcceptArgs = new SocketAsyncEventArgs();
        public TcpPolicyServer()
        {
            Console.WriteLine(System.Environment.CommandLine);
            FileStream fs = new FileStream(TcpPolicyServer.PolicyFileName, FileMode.Open);
            _policyBuffer = new byte[fs.Length];
            fs.Read(_policyBuffer, 0, _policyBuffer.Length);
            _socketAcceptArgs.Completed += new EventHandler<SocketAsyncEventArgs>(AcceptAsyncCallback);
        }
        public void Start(int port)
        {
            //read the policy file into the buffer
            IPHostEntry ipHostInfo = Dns.GetHostEntry(Dns.GetHostName());
            //Should be within the port range of 4502-4532
            IPEndPoint localEndPoint = new IPEndPoint(IPAddress.Any, port);

            _listener = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

            // Bind the socket to the local endpoint and listen for incoming connections.
            try
            {
                _listener.Bind(localEndPoint);
                _listener.Listen(50);
                AcceptAsync();
            }
            catch (Exception e)
            {
                throw new AppServerException("Can't start policy server", e);
            }
        }

        public void Stop()
        {
            _listener.Close();
        }
        void AcceptAsync()
        {
            AcceptAsync(_socketAcceptArgs);
        }

        void AcceptAsync(SocketAsyncEventArgs socketAcceptArgs)
        {

            if (!_listener.AcceptAsync(socketAcceptArgs))
            {
                AcceptAsyncCallback(socketAcceptArgs.AcceptSocket, socketAcceptArgs);
            }
        }

        void AcceptAsyncCallback(object sender, SocketAsyncEventArgs e)
        {
            if (e.SocketError == SocketError.Success)
            {
                Console.WriteLine("Connection accepted...");
                TcpClientConnection con = new TcpClientConnection(e.AcceptSocket, this._policyBuffer);
                con.ReceiveAsync();
            }
            //the following is necessary for reuse of the _socketAcceptArgs
            e.AcceptSocket = null;
            //schedule a new accept request
            AcceptAsync();
        }
    }
}
